@@ -38,24 +38,19 @@ class BdistMSI(bdist_msi):
             None,
             "temporary directory for creating the distribution",
         ),
-        (
-            "keep-temp",
-            "k",
-            "keep the pseudo-installation tree around after "
-            "creating the distribution archive",
-        ),
         ("dist-dir=", "d", "directory to put final built distributions in"),
-        (
-            "skip-build",
-            None,
-            "skip rebuilding everything (for testing/debugging)",
-        ),
         (
             "install-script=",
             None,
             "basename of installation script to be run after "
             "installation or before deinstallation",
         ),
+        (
+            "keep-temp",
+            "k",
+            "keep the pseudo-installation tree around after "
+            "creating the distribution archive",
+        ),
         (
             "pre-install-script=",
             None,
@@ -63,29 +58,35 @@ class BdistMSI(bdist_msi):
             "any files are installed.  This script need not be in the "
             "distribution",
         ),
+        (
+            "skip-build",
+            None,
+            "skip rebuilding everything (for testing/debugging)",
+        ),
         # cx_Freeze specific
         ("add-to-path=", None, "add target dir to PATH environment variable"),
-        ("upgrade-code=", None, "upgrade code to use"),
-        ("initial-target-dir=", None, "initial target directory"),
-        ("target-name=", None, "name of the file to create"),
-        ("directories=", None, "list of 3-tuples of directories to create"),
-        ("environment-variables=", None, "list of environment variables"),
+        ("all-users=", None, "installation for all users (or just me)"),
         (
             "data=",
             None,
             "dictionary of data indexed by table name, and each value is a "
             "tuple to include in table",
         ),
+        ("directories=", None, "list of 3-tuples of directories to create"),
+        ("environment-variables=", None, "list of environment variables"),
+        ("extensions=", None, "Extensions for which to register Verbs"),
+        ("initial-target-dir=", None, "initial target directory"),
+        ("install-icon=", None, "icon path to add/remove programs "),
+        ("product-code=", None, "product code to use"),
         (
             "summary-data=",
             None,
             "Dictionary of data to include in msi summary data stream. "
             'Allowed keys are "author", "comments", "keywords".',
         ),
-        ("product-code=", None, "product code to use"),
-        ("install-icon=", None, "icon path to add/remove programs "),
-        ("all-users=", None, "installation for all users (or just me)"),
-        ("extensions=", None, "Extensions for which to register Verbs"),
+        ("target-name=", None, "name of the file to create"),
+        ("target-version=", None, "version of the file to create"),
+        ("upgrade-code=", None, "upgrade code to use"),
     ]
 
     boolean_options = [
@@ -854,6 +855,8 @@ class BdistMSI(bdist_msi):
         self.add_to_path = None
         self.initial_target_dir = None
         self.target_name = None
+        self.target_version = None
+        self.fullname = None
         self.directories = None
         self.environment_variables = None
         self.data = None
@@ -888,8 +891,13 @@ class BdistMSI(bdist_msi):
         self.install_script_key = None
 
         # cx_Freeze specific
-        name = self.distribution.get_name()
-        fullname = self.distribution.get_fullname()
+        if self.target_name is None:
+            self.target_name = self.distribution.get_name()
+        if self.target_version is None and self.distribution.metadata.version:
+            self.target_version = self.distribution.metadata.version
+        name = self.target_name
+        version = self.target_version or self.distribution.get_version()
+        self.fullname = f"{name}-{version}"
         platform = get_platform().replace("win-amd64", "win64")
         if self.initial_target_dir is None:
             if platform == "win64" or platform.startswith("mingw_x86_64"):
@@ -899,12 +907,6 @@ class BdistMSI(bdist_msi):
             self.initial_target_dir = rf"[{program_files_folder}]\{name}"
         if self.add_to_path is None:
             self.add_to_path = False
-        if self.target_name is None:
-            self.target_name = fullname
-        if not self.target_name.lower().endswith(".msi"):
-            self.target_name = f"{self.target_name}-{platform}.msi"
-        if not os.path.isabs(self.target_name):
-            self.target_name = os.path.join(self.dist_dir, self.target_name)
         if self.directories is None:
             self.directories = []
         if self.environment_variables is None:
@@ -939,21 +941,19 @@ class BdistMSI(bdist_msi):
                     "Executable must be the base target name of one of the "
                     "distribution's executables"
                 ) from None
-            distribution_name = self.distribution.get_name()
             stem = os.path.splitext(executable)[0]
-            version = self.distribution.get_version()
-            progid = msilib.make_id(f"{distribution_name}.{stem}.{version}")
+            progid = msilib.make_id(f"{name}.{stem}.{version}")
             mime = extension.get("mime", None)
             # "%1" a better default for argument?
             argument = extension.get("argument", None)
-            context = extension.get("context", f"{fullname} {verb}")
+            context = extension.get("context", f"{self.fullname} {verb}")
             # Add rows via self.data to safely ignore duplicates
             self._append_to_data(
                 "ProgId",
                 progid,
                 None,
                 None,
-                self.distribution.get_description(),
+                self.distribution.get_description() or "UNKNOWN",
                 None,
                 None,
             )
@@ -970,7 +970,7 @@ class BdistMSI(bdist_msi):
                 -1,
                 rf"Software\Classes\{progid}",
                 "FriendlyAppName",
-                self.distribution.get_name(),
+                name,
                 component,
             )
             self._append_to_data(
@@ -979,7 +979,7 @@ class BdistMSI(bdist_msi):
                 -1,
                 rf"Software\Classes\{progid}\shell\{verb}",
                 "FriendlyAppName",
-                self.distribution.get_name(),
+                name,
                 component,
             )
             self._append_to_data(
@@ -988,28 +988,39 @@ class BdistMSI(bdist_msi):
                 -1,
                 rf"Software\Classes\{progid}\Application",
                 "ApplicationCompany",
-                self.distribution.get_author(),
+                self.distribution.get_author() or "UNKNOWN",
                 component,
             )
 
     def run(self):
         if not self.skip_build:
             self.run_command("build")
+
+        # install everything from build directory in a new prefix
+        install_dir = self.bdist_dir
         install = self.reinitialize_command("install", reinit_subcommands=1)
-        bdist_dir = self.bdist_dir
-        install.prefix = bdist_dir
+        install.prefix = install_dir
         install.skip_build = self.skip_build
         install.warn_dir = 0
-        logging.info("installing to %s", bdist_dir)
+        logging.info("installing to %s", install_dir)
         install.ensure_finalized()
         install.run()
+
+        # make msi (by default in dist directory)
         self.mkpath(self.dist_dir)
-        # fullname = self.distribution.get_fullname()
-        if os.path.exists(self.target_name):
-            os.unlink(self.target_name)
-        metadata = self.distribution.metadata
-        author = metadata.get_contact()
-        version = metadata.get_version()
+        platform = get_platform().replace("win-amd64", "win64")
+        if self.target_version:
+            base_name = f"{self.fullname}-{platform}.msi"
+        else:
+            base_name = f"{self.target_name}-{platform}.msi"
+        installer_name = os.path.join(self.dist_dir, base_name)
+        installer_name = os.path.abspath(installer_name)
+        if os.path.exists(installer_name):
+            os.unlink(installer_name)
+
+        author = self.distribution.metadata.get_contact() or "UNKNOWN"
+        version = self.target_version or self.distribution.get_version()
+        # ProductVersion must be strictly numeric
         base_version = Version(version).base_version
 
         # msilib is reloaded in order to reset the "_directories" global member
@@ -1024,9 +1035,9 @@ class BdistMSI(bdist_msi):
         if self.product_code is None:
             self.product_code = msilib.gen_uuid()
         self.db = msilib.init_database(
-            self.target_name,
+            installer_name,
             msilib.schema,
-            self.distribution.metadata.name,
+            self.target_name,
             self.product_code,
             base_version,
             author,
@@ -1043,12 +1054,14 @@ class BdistMSI(bdist_msi):
         )
 
         if not self.keep_temp:
-            logging.info("removing '%s' (and everything under it)", bdist_dir)
+            logging.info(
+                "removing '%s' (and everything under it)", install_dir
+            )
             if not self.dry_run:
                 try:
-                    shutil.rmtree(bdist_dir)
+                    shutil.rmtree(install_dir)
                 except OSError as exc:
-                    logging.warning("error removing %s: %s", bdist_dir, exc)
+                    logging.warning("error removing %s: %s", install_dir, exc)
 
         # Cause the MSI file to be released. Without this, then if bdist_msi
         # is run programmatically from within a larger script, subsequent
